import sys

import walden.cli as walden

def main(args=None):
    # TODO: use arg parse instead?
    if len(sys.argv) == 1 or sys.argv[1] == '-h':
        walden.show_help()
        sys.exit(0)
    else:
        walden.main(sys.argv[1:])


if __name__ == "__main__":
    sys.exit(-1)



#!/usr/bin/env python3
# library imports
import sys
import pathlib
import pickle
import shutil
import subprocess
import signal

# package imports
import walden.compiler as compiler
import walden.journal as journal
from walden.utils import constants


DEFAULT_JOURNALS_PATH = constants.DEFAULT_JOURNALS_PATH

def term_handler(signum, frame):
    #TODO: maybe this can be improved?
    exit(-1)

signal.signal(signal.SIGINT, term_handler)

def check_and_create_journals_folder():
    if not pathlib.Path(DEFAULT_JOURNALS_PATH).exists():
        pathlib.Path(DEFAULT_JOURNALS_PATH).mkdir()
        pickle.dump( {}, open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'wb'))


def show_help():
    message = '''
    -h: show this dialog
    init <journal_name>?: create new journal
    delete <journal_name>: delete an existing journal
    today <journal_name>: new entry in specified journal
    list: list names of journals
    build <journal_name>: compile journal into .pdf
    view <journal_name>: view journal as .pdf
    '''
    print(message)



# save journal name and path to .journals_info file
# journal_info = (name, path)
def save_journal_info(journal_info):
    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )
    meta_data[journal_info[0]] = journal_info[1]
    pickle.dump(meta_data, open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'wb'))
    return journal_info[0]


def delete_journal(journal_name):
    if journal_name is None:
        return 'No journal specified. See walden -h for help.'

    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )

    if journal_name in meta_data:
        confirmation = ''
        while not (confirmation.lower() == 'y' or confirmation.lower() == 'n'):
            confirmation = input(f'Are you sure you want to delete "{journal_name}"? (Y/N): ')

        if confirmation.casefold() == 'y'.casefold():
            path = pathlib.Path(meta_data.pop(journal_name))
            pickle.dump(meta_data, open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'wb'))
            shutil.rmtree(path)
            return f'journal "{journal_name}" deleted.'
        else:
            return f'"{journal_name}" not deleted.'

    else:
        return f'No journal named "{journal_name}" exists. See walden -list for list of journals.'


def process_today(journal_name):
    if journal_name is None:
        print('No journal specified. See walden -h for help.')
        return

    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )

    if journal_name in meta_data:
        journal.edit_today(meta_data[journal_name])
    else:
        print(f'No journal named "{journal_name}" exists. See walden -list for list of journals.')


def list_journals():
    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )

    journal_names = ['Journals:']
    for name in meta_data:
        journal_names.append(name)

    return '\n'.join(journal_names)


def process_build(journal_name):
    if journal_name is None:
        print('No journal specified. See walden -h for help.')
        return

    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )

    if journal_name in meta_data:
        compiler.compile_journal(meta_data[journal_name])
    else:
        print(f'No journal named "{journal_name}" exists. See walden -list for list of journals.')


def process_view(journal_name):
    if journal_name is None:
        print('No journal specified. See walden -h for help.')
        return

    meta_data = pickle.load( open(f'{DEFAULT_JOURNALS_PATH}/.journals_info', 'rb') )

    if journal_name in meta_data:
        if not pathlib.Path(f'{meta_data[journal_name]}/log.pdf').exists():
            compiler.compile_journal(meta_data[journal_name])
        subprocess.call(['open', f'{meta_data[journal_name]}/log.pdf'])
    else:
        print(f'No journal named "{journal_name}" exists. See walden -list for list of journals.')

#TODO: allow changing path where journals are stored
#def process_set_path(new_journal_path):
#    if new_journal_path is None:
#        print(f'No new journal path specified. See walden -h for help.')
#
#    new_journal_path = 



def main(args):
    check_and_create_journals_folder()

    command = args[0]
    argument = args[1] if 1 < len(args) else None

    if command == 'init':
        journal_info = journal.init(argument)
        journal_name = save_journal_info(journal_info)
        print(f'{journal_name} created!')
    elif command == '-h':
        show_help()
    elif command == 'delete':
        response = delete_journal(argument)
        print(response)
    elif command == 'list':
        names = list_journals()
        print(names)
    elif command == 'today':
        res = process_today(argument)
    elif command == 'build':
        process_build(argument)
    elif command == 'view':
        process_view(argument)
    #elif command == 'set-path':
    #    process_set_path(argument)
    else:
        print(f'Invalid command "{command}". See walden -h for help.')



#!/usr/bin/env python3
import pathlib
import datetime
from pathlib import Path

from walden.utils import constants

DEFAULT_JOURNALS_PATH = constants.DEFAULT_JOURNALS_PATH


def generate_folders(journal_path):
    today = datetime.datetime.now()
    if today.month > 9:
        pathlib.Path(f'{journal_path}/entries/{today.year}/{today.month}').mkdir(parents=True)
    else:
        pathlib.Path(f'{journal_path}/entries/{today.year}/0{today.month}').mkdir(parents=True)


def gen_new_aux_page(label, is_title=False):
    page = []

    if is_title:
        page.append("\\thispagestyle{empty}")

    page.append("\\begin{center}")
    page.append("\t\\vfil")
    page.append("\t\\vspace*{0.4\\textheight}\n")
    page.append("\t\\Huge")
    page.append(f"\t\\bf{{{label}}}\n")
    page.append("\t\\normalsize")
    page.append("\\end{center}")

    return "\n".join(page)


#TODO: maybe journal name can be passed in now?
def generate_resources(path, journal_name="journal"):
    files = [("newmonth.tex", "\\month"),
             ("newyear.tex", "\\year"),
             ("title.tex", journal_name)]

    Path(f"{path}/aux").mkdir()

    for file in files:
        with open(f"{path}/aux/{file[0]}", "w") as new_aux:
            new_aux.write(gen_new_aux_page(file[1], "title" in file[1]))


def init(name):
    if name is None:
        name = input('Enter desired journal name: ')

    path = f'{DEFAULT_JOURNALS_PATH}/{name}'

    while (pathlib.Path(path).exists()):
        print('Sorry, a journal with that name already exists')
        name = input('Enter desired journal name: ')
        path = f'{DEFAULT_JOURNALS_PATH}/{name}'

    pathlib.Path(path).mkdir()

    #create folder structure
    generate_resources(path, name)
    generate_folders(path)

    return (name, path)



#/usr/bin/env python3
import pathlib
import os
from textwrap import dedent
import subprocess
import shutil

from .log_generator import generate_log

# compiles the journal using pdflatex
def compile_journal(journal_path):
    entries = parse_entries(f'{journal_path}/entries')
    compile_months(f'{journal_path}/entries', entries)
    generate_log(journal_path, entries)

    #TODO: Make this cleaner using context managing 
    current_cwd = pathlib.Path.cwd()
    os.chdir(f'{journal_path}')

    # compile journal 
    pathlib.Path('build').mkdir(exist_ok=True)
    subprocess.call(['pdflatex', '-output-directory=build', 'log.tex'])

    #cleanup aux files
    pathlib.Path('build/log.pdf').replace('log.pdf')
    shutil.rmtree(pathlib.Path('build'))

    #TODO cleanup misc monthly files
    os.chdir(current_cwd)



# get months within a year
def parse_entries(path):
    entries = dict()

    years_folder = pathlib.Path(path)

    years = [str(f)[str(f).rindex('/')+1:] for f in years_folder.iterdir() if f.is_dir()]
    years.sort()

    for year in years:
        months_folder = pathlib.Path(f'{path}/{year}')
        entries[year] = [str(f)[str(f).rindex('/')+1:] for f in months_folder.iterdir() if f.is_dir()]
        entries[year].sort()

    return entries


def compile_months(entries_path, entries):
    for year in entries:
        for month in entries[year]:
            with open(f'{entries_path}/{year}/{month}.tex', 'w') as monthly_entry:
                monthly_entry.write(dedent(format_month(f'{entries_path}/{year}/{month}', year, month)))


def format_month(month_path, year, month):
    month_folder = pathlib.Path(month_path)
    daily_entries = [str(f)[str(f).rindex('/')+1:] for f in month_folder.iterdir() if f.is_file()]
    daily_entries.sort()

    formatted_entries = []
    for entry in daily_entries:
        formatted_entries.append(f'\\input{{entries/{year}/{month}/{entry.strip()}}}')

    return '\n'.join(formatted_entries)



if __name__ == '__main__':
    compile_journal('../journals/a')





#!/usr/bin/env python3
import pathlib
from textwrap import dedent

MONTH_NAME = {
        '01': 'January',
        '02': 'February',
        '03': 'March',
        '04': 'April',
        '05': 'May',
        '06': 'June',
        '07': 'July',
        '08': 'August',
        '09': 'September',
        '10': 'October',
        '11': 'November',
        '12': 'December'
        }

# path: path to root of journal
def generate_log(path, entries):
    headers = initialize_header()
    document = initialize_document(entries)

    with open(f'{path}/log.tex', 'w') as log:
        log.write(dedent(headers))
        log.write(document)


def initialize_header():
    return r'''
    \documentclass[12pt]{article}
    \usepackage{graphicx}

    \setlength\evensidemargin{0.0in}
    \setlength\oddsidemargin{0.0in}
    \setlength\textwidth{6.5in}
    \setlength\textheight{9.5in}
    \setlength\topmargin{-0.5in}

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % For Footer
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    \usepackage[english]{babel}
    \usepackage[utf8]{inputenc}
    \usepackage{fancyhdr}

    \pagestyle{fancy}
    \fancyhf{}
    \renewcommand{\headrulewidth}{0pt}
    %\rhead{Share\LaTeX}
    %\lhead{Guides and tutorials}
    \rfoot{\thepage}
    \def\day{}
    \cfoot{\textit{\day}}

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % For Multiple Columns within Text
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    \usepackage{multicol}

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % For Links to Websites
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    \usepackage{hyperref}
    \hypersetup{
        colorlinks=true,
        linkcolor=blue,
        filecolor=magenta,
        urlcolor=cyan,
    }

    \urlstyle{same}

    '''


# compile years and months into latex form
def initialize_document(entries):
    #entries = parse_entries(f'{path}/entries')
    document = []
    document.append('\\begin{document}\n')
    document.append('\\input{aux/title}\n\\clearpage\n')

    for year in entries:
        document.append(dedent(format_year(year)))
        for month in entries[year]:
            document.append(dedent(format_month(year, month)))
        document.append('\\clearpage')

    document.append('\n\\end{document}\n')
    return ''.join(document)


def format_year(year):
    return f'''
    \\def\\year{{{year}}}
    \\input{{aux/newyear}}
    \\clearpage
    '''


def format_month(year, month):
    return f'''
    \\def\\month{{{MONTH_NAME[month]}}}
    \\input{{aux/newmonth}}
    \\clearpage
    \\input{{entries/{year}/{month}}}
    '''



import setuptools

with open("README.md", "r") as readme:
    long_description = readme.read()

setuptools.setup(
    name="walden",
    version="0.0.71",
    author="Aravind Koneru",
    author_email="aravind.b.koneru@gmail.com",
    description="A .tex journaling system",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aravindkoneru/Walden",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points = {
        "console_scripts": ["walden=walden.__main__:main"]
    },
    python_requires=">=3.5"
)



#!/usr/bin/env python3
import pathlib
import datetime
import subprocess
import os

EDITOR = os.environ.get('EDITOR', 'vim')

def edit_today(journal_path):
    today = datetime.datetime.now()
    entry_path = None

    if today.month > 9:
        entry_path = pathlib.Path(f"{journal_path}/entries/{today.year}/{today.month}")
    else:
        entry_path = pathlib.Path(f"{journal_path}/entries/{today.year}/0{today.month}")

    entry_path.mkdir(parents=True, exist_ok=True)
    entry_path = entry_path / f"{today.day}.tex"

    if not entry_path.is_file():
        create_entry(entry_path)

    subprocess.call([EDITOR, entry_path])


def create_entry(entry_path):
    entry = []
    today = datetime.date.today()

    print('Creating today\'s journal entry...')

    entry.append(today.strftime("\\def\\day{\\textit{%B %d, %Y}}"))
    entry.append(today.strftime("\\def\\weekday{\\textit{%A}}"))
    entry.append("\\subsection*{\\weekday, \\day}\n\n")

    with open(entry_path, "w") as new_entry:
        new_entry.write('\n'.join(entry))


if __name__ == '__main__':
    edit_today('journals/a')
